@@ -77,45 +77,32 @@ class RollerTrolDevice(RFXtrxDevice):
             self.id_combined = pkt.id_combined
             self.unitcode = pkt.unitcode
             self.cmndseqnbr = 0
+            self.COMMANDS = lowlevel.RollerTrol.COMMANDS
+
+    def send_command(self, transport, command):
+        """ Send a command using the given transport """
+        pkt = lowlevel.RollerTrol()
+        pkt.set_transmit(
+            self.subtype,
+            self.cmndseqnbr,
+            self.id_combined,
+            self.unitcode,
+            command
+        )
+        self.cmndseqnbr = (self.cmndseqnbr + 1) % 5
+        transport.send(pkt.data)
 
     def send_close(self, transport):
         """ Send a 'Close' command using the given transport """
-        pkt = lowlevel.RollerTrol()
-        pkt.set_transmit(
-            self.subtype,
-            self.cmndseqnbr,
-            self.id_combined,
-            self.unitcode,
-            0x01
-        )
-        self.cmndseqnbr = (self.cmndseqnbr + 1) % 5
-        transport.send(pkt.data)
+        self.send_command(transport, 0x01)
 
     def send_open(self, transport):
         """ Send an 'Open' command using the given transport """
-        pkt = lowlevel.RollerTrol()
-        pkt.set_transmit(
-            self.subtype,
-            self.cmndseqnbr,
-            self.id_combined,
-            self.unitcode,
-            0x00
-        )
-        self.cmndseqnbr = (self.cmndseqnbr + 1) % 5
-        transport.send(pkt.data)
+        self.send_command(transport, 0x00)
 
     def send_stop(self, transport):
         """ Send a 'Stop' command using the given transport """
-        pkt = lowlevel.RollerTrol()
-        pkt.set_transmit(
-            self.subtype,
-            self.cmndseqnbr,
-            self.id_combined,
-            self.unitcode,
-            0x02
-        )
-        self.cmndseqnbr = (self.cmndseqnbr + 1) % 5
-        transport.send(pkt.data)
+        self.send_command(transport, 0x02)
 
 
 class RfyDevice(RFXtrxDevice):
@@ -127,71 +114,56 @@ class RfyDevice(RFXtrxDevice):
             self.id_combined = pkt.id_combined
             self.unitcode = pkt.unitcode
             self.cmndseqnbr = 0
+            self.COMMANDS = lowlevel.Rfy.COMMANDS
+
+    def send_command(self, transport, command):
+        """ Send a command using the given transport """
+        pkt = lowlevel.Rfy()
+        pkt.set_transmit(
+            self.subtype,
+            self.cmndseqnbr,
+            self.id_combined,
+            self.unitcode,
+            command
+        )
+        self.cmndseqnbr = (self.cmndseqnbr + 1) % 5
+        transport.send(pkt.data)
 
     def send_close(self, transport):
         """ Send a 'Close' command using the given transport """
-        pkt = lowlevel.Rfy()
-        pkt.set_transmit(
-            self.subtype,
-            self.cmndseqnbr,
-            self.id_combined,
-            self.unitcode,
-            0x03
-        )
-        self.cmndseqnbr = (self.cmndseqnbr + 1) % 5
-        transport.send(pkt.data)
+        self.send_command(transport, 0x03)
 
     def send_open(self, transport):
         """ Send an 'Open' command using the given transport """
-        pkt = lowlevel.Rfy()
-        pkt.set_transmit(
-            self.subtype,
-            self.cmndseqnbr,
-            self.id_combined,
-            self.unitcode,
-            0x01
-        )
-        self.cmndseqnbr = (self.cmndseqnbr + 1) % 5
-        transport.send(pkt.data)
+        self.send_command(transport, 0x01)
 
     def send_stop(self, transport):
         """ Send a 'Stop' command using the given transport """
-        pkt = lowlevel.Rfy()
-        pkt.set_transmit(
-            self.subtype,
-            self.cmndseqnbr,
-            self.id_combined,
-            self.unitcode,
-            0x00
-        )
-        self.cmndseqnbr = (self.cmndseqnbr + 1) % 5
-        transport.send(pkt.data)
+        self.send_command(transport, 0x00)
 
     def send_on(self, transport):
         """ Send an 'Enable Sun Automation' command """
-        pkt = lowlevel.Rfy()
-        pkt.set_transmit(
-            self.subtype,
-            self.cmndseqnbr,
-            self.id_combined,
-            self.unitcode,
-            0x13
-        )
-        self.cmndseqnbr = (self.cmndseqnbr + 1) % 5
-        transport.send(pkt.data)
+        self.send_command(transport, 0x13)
 
     def send_off(self, transport):
         """ Send an 'Disable Sun Automation' command """
-        pkt = lowlevel.Rfy()
-        pkt.set_transmit(
-            self.subtype,
-            self.cmndseqnbr,
-            self.id_combined,
-            self.unitcode,
-            0x14
-        )
-        self.cmndseqnbr = (self.cmndseqnbr + 1) % 5
-        transport.send(pkt.data)
+        self.send_command(transport, 0x14)
+
+    def send_up05sec(self, transport):
+        """ Send a '0.5 Seconds Up' command """
+        self.send_command(transport, 0x0F)
+
+    def send_down05sec(self, transport):
+        """ Send a '0.5 Seconds Down' command """
+        self.send_command(transport, 0x10)
+
+    def send_up2sec(self, transport):
+        """ Send a '2 Seconds Up' command """
+        self.send_command(transport, 0x11)
+
+    def send_down2sec(self, transport):
+        """ Send a '2 Seconds Down' command """
+        self.send_command(transport, 0x12)
 
 
 class LightingDevice(RFXtrxDevice):
@@ -203,62 +175,94 @@ class LightingDevice(RFXtrxDevice):
         if isinstance(pkt, lowlevel.Lighting1):
             self.housecode = pkt.housecode
             self.unitcode = pkt.unitcode
+            self.COMMANDS = lowlevel.Lighting1.COMMANDS
         if isinstance(pkt, lowlevel.Lighting2):
             self.id_combined = pkt.id_combined
             self.unitcode = pkt.unitcode
+            self.COMMANDS = lowlevel.Lighting2.COMMANDS
         if isinstance(pkt, lowlevel.Lighting3):
             self.system = pkt.system
             self.channel = pkt.channel
+            self.COMMANDS = lowlevel.Lighting3.COMMANDS
         if isinstance(pkt, lowlevel.Lighting4):
             self.cmd = pkt.cmd
             self.pulse = pkt.pulse
+            self.COMMANDS = lowlevel.Lighting4.COMMANDS
         if isinstance(pkt, lowlevel.Lighting5):
             self.id_combined = pkt.id_combined
             self.unitcode = pkt.unitcode
+            if self.subtype == 0x00:
+                self.COMMANDS = lowlevel.Lighting5.COMMANDS_00
+            elif self.subtype == 0x01:
+                self.COMMANDS = lowlevel.Lighting5.COMMANDS_01
+            elif self.subtype in (0x02, 0x04, 0x0F):
+                self.COMMANDS = lowlevel.Lighting5.COMMANDS_02_04_0F
+            elif self.subtype == 0x03:
+                self.COMMANDS = lowlevel.Lighting5.COMMANDS_03
+            else:
+                self.COMMANDS = lowlevel.Lighting5.COMMANDS_XX
         if isinstance(pkt, lowlevel.Lighting6):
             self.id_combined = pkt.id_combined
             self.groupcode = pkt.groupcode
             self.unitcode = pkt.unitcode
             self.cmndseqnbr = 0
+            self.COMMANDS = lowlevel.Lighting6.COMMANDS
 
-    def send_onoff(self, transport, turn_on):
-        """ Send an 'On' or 'Off' command using the given transport """
+    def send_command(self, transport, command):
+        """ Send an ommand using the given transport """
         if self.packettype == 0x10:  # Lighting1
             pkt = lowlevel.Lighting1()
             pkt.set_transmit(self.subtype, 0, self.housecode, self.unitcode,
-                             turn_on and 0x01 or 0x00)
+                             command)
             transport.send(pkt.data)
         elif self.packettype == 0x11:  # Lighting2
             pkt = lowlevel.Lighting2()
             pkt.set_transmit(self.subtype, 0, self.id_combined, self.unitcode,
-                             turn_on and 0x01 or 0x00, 0x00)
+                             command, 0x00)
             transport.send(pkt.data)
         elif self.packettype == 0x12:  # Lighting3
             pkt = lowlevel.Lighting3()
             pkt.set_transmit(self.subtype, 0, self.system, self.channel,
-                             turn_on and 0x10 or 0x1a)
+                             command)
             transport.send(pkt.data)
         elif self.packettype == 0x13:  # Lighting4
             pkt = lowlevel.Lighting4()
             code = self.cmd & ~1
-            code |= 0x1 if turn_on else 0x0
+            code |= command
             pkt.set_transmit(self.subtype, 0, code, self.pulse)
             transport.send(pkt.data)
         elif self.packettype == 0x14:  # Lighting5
             pkt = lowlevel.Lighting5()
             pkt.set_transmit(self.subtype, 0, self.id_combined, self.unitcode,
-                             turn_on and 0x01 or 0x00, 0x00)
+                             command, 0x00)
             transport.send(pkt.data)
         elif self.packettype == 0x15:  # Lighting6
             pkt = lowlevel.Lighting6()
             pkt.set_transmit(self.subtype, 0, self.id_combined, self.groupcode,
                              self.unitcode,
-                             not turn_on and 0x01 or 0x00, self.cmndseqnbr)
+                             command, self.cmndseqnbr)
             self.cmndseqnbr = (self.cmndseqnbr + 1) % 5
             transport.send(pkt.data)
         else:
             return
 
+    def send_onoff(self, transport, turn_on):
+        """ Send an 'On' or 'Off' command using the given transport """
+        if self.packettype == 0x10:  # Lighting1
+            self.send_command(transport, turn_on and 0x01 or 0x00)
+        elif self.packettype == 0x11:  # Lighting2
+            self.send_command(transport, turn_on and 0x01 or 0x00)
+        elif self.packettype == 0x12:  # Lighting3
+            self.send_command(transport, turn_on and 0x10 or 0x1a)
+        elif self.packettype == 0x13:  # Lighting4
+            self.send_command(transport, 0x1 if turn_on else 0x0)
+        elif self.packettype == 0x14:  # Lighting5
+            self.send_command(transport, turn_on and 0x01 or 0x00)
+        elif self.packettype == 0x15:  # Lighting6
+            self.send_command(transport, not turn_on and 0x01 or 0x00)
+        else:
+            return
+
     def send_on(self, transport):
         """ Send an 'On' command using the given transport """
         self.send_onoff(transport, True)
@@ -273,10 +277,7 @@ class LightingDevice(RFXtrxDevice):
         if self.packettype == 0x14:  # Lighting5
             if command not in [0x0d, 0x0e, 0x0f]:
                 raise ValueError(command, "is not a relay packet in Lighting5")
-            pkt = lowlevel.Lighting5()
-            pkt.set_transmit(self.subtype, 0, self.id_combined, self.unitcode,
-                             command, 0x00)
-            transport.send(pkt.data)
+            self.send_command(transport, command)
         else:
             raise ValueError("Unsupported packettype")
 
@@ -344,60 +345,72 @@ class ChimeDevice(RFXtrxDevice):
         super().__init__(pkt)
         self.id1 = pkt.id1
         self.id2 = pkt.id2
+        self.COMMANDS = lowlevel.Chime.COMMANDS
 
-    def send_chime(self, transport, sound):
+    def send_command(self, transport, sound):
         """Trigger a chime sound on device."""
         pkt = lowlevel.Chime()
         pkt.set_transmit(self.subtype, 0, self.id1, self.id2, sound)
         transport.send(pkt.data)
 
+###############################################################################
+# get_device_from_pkt method
+###############################################################################
+
+
+def get_device_from_pkt(pkt):
+    """Construct a device object from a packet."""
+    # pylint: disable=too-many-boolean-expressions
+    if isinstance(pkt, (lowlevel.Lighting1, lowlevel.Lighting2,
+                        lowlevel.Lighting3, lowlevel.Lighting4,
+                        lowlevel.Lighting5, lowlevel.Lighting6)):
+        device = LightingDevice(pkt)
+    elif isinstance(pkt, lowlevel.RollerTrol):
+        device = RollerTrolDevice(pkt)
+    elif isinstance(pkt, lowlevel.Rfy):
+        device = RfyDevice(pkt)
+    elif isinstance(pkt, lowlevel.Chime):
+        device = ChimeDevice(pkt)
+    elif isinstance(pkt, lowlevel.Security1):
+        device = SecurityDevice(pkt)
+    else:
+        device = RFXtrxDevice(pkt)
+    return device
+
+
+class SecurityDevice(RFXtrxDevice):
+    """ Concrete class for a control device """
+
+    def __init__(self, pkt):
+        super().__init__(pkt)
+        self.id_combined = pkt.id_combined
+        self.cmndseqnbr = 0
+        self.STATUS = lowlevel.Security1.STATUS
+
+    def send_status(self, transport, status):
+        """Trigger a status message on device."""
+        pkt = lowlevel.Security1()
+        pkt.set_transmit(
+            self.subtype,
+            self.cmndseqnbr,
+            self.id_combined,
+            status
+        )
+        self.cmndseqnbr = (self.cmndseqnbr + 1) % 5
+        transport.send(pkt.data)
+
 
 ###############################################################################
-# get_devide method
+# get_device method
 ###############################################################################
 
 
 def get_device(packettype, subtype, id_string):
     """ Return a device base on its identifying values """
-    # pylint: disable=too-many-return-statements
-    if packettype == 0x10:  # Lighting1
-        pkt = lowlevel.Lighting1()
-        pkt.parse_id(subtype, id_string)
-        return LightingDevice(pkt)
-    if packettype == 0x11:  # Lighting2
-        pkt = lowlevel.Lighting2()
-        pkt.parse_id(subtype, id_string)
-        return LightingDevice(pkt)
-    if packettype == 0x12:  # Lighting3
-        pkt = lowlevel.Lighting3()
-        pkt.parse_id(subtype, id_string)
-        return LightingDevice(pkt)
-    if packettype == 0x13:  # Lighting4
-        pkt = lowlevel.Lighting4()
-        pkt.parse_id(subtype, id_string)
-        return LightingDevice(pkt)
-    if packettype == 0x14:  # Lighting5
-        pkt = lowlevel.Lighting5()
-        pkt.parse_id(subtype, id_string)
-        return LightingDevice(pkt)
-    if packettype == 0x15:  # Lighting6
-        pkt = lowlevel.Lighting6()
-        pkt.parse_id(subtype, id_string)
-        return LightingDevice(pkt)
-    if packettype == 0x16:  # Chime
-        pkt = lowlevel.Chime()
-        pkt.parse_id(subtype, id_string)
-        return ChimeDevice(pkt)
-    if packettype == 0x19:  # RollerTrol
-        pkt = lowlevel.RollerTrol()
-        pkt.parse_id(subtype, id_string)
-        return RollerTrolDevice(pkt)
-    if packettype == 0x1A:  # Rfy
-        pkt = lowlevel.Rfy()
-        pkt.parse_id(subtype, id_string)
-        return RfyDevice(pkt)
-
-    raise ValueError("Unsupported packettype")
+    pkt = lowlevel.get_packet_with_id(packettype, subtype, id_string)
+    if pkt is None:
+        raise ValueError("Unsupported packettype")
+    return get_device_from_pkt(pkt)
 
 
 ###############################################################################
@@ -420,7 +433,7 @@ class SensorEvent(RFXtrxEvent):
 
     def __init__(self, pkt):
         #  pylint: disable=too-many-branches, too-many-statements
-        device = RFXtrxDevice(pkt)
+        device = get_device_from_pkt(pkt)
         super().__init__(device)
 
         self.values = {}
@@ -481,6 +494,20 @@ class SensorEvent(RFXtrxEvent):
             self.values['Current'] = pkt.currentamps
             self.values['Energy usage'] = pkt.currentwatt
             self.values['Total usage'] = pkt.totalwatthours
+        if isinstance(pkt, lowlevel.Cartelectronic):
+            if pkt.type_string == 'CARTELECTRONIC_ENCODER':
+                self.values['Counter value'] = pkt.counter1
+                self.values['Count'] = pkt.counter2
+            elif pkt.type_string == 'CARTELECTRONIC_LINKY':
+                # Index for current tarif if consummer
+                self.values['Total usage'] = pkt.conswatthours
+                # Index for current tarif if production
+                self.values['Count'] = pkt.prodwatthours
+                # Index of current tarif
+                self.values['Counter value'] = pkt.tarif_num
+                self.values['Voltage'] = pkt.voltage
+                self.values['Energy usage'] = pkt.currentwatt
+                self.values['Sensor Status'] = pkt.teleinfo_ok
         if isinstance(pkt, lowlevel.Security1):
             self.values['Sensor Status'] = pkt.security1_status_string
         if not isinstance(pkt, (lowlevel.Energy5, lowlevel.RfxMeter)):
@@ -500,19 +527,7 @@ class ControlEvent(RFXtrxEvent):
     """ Concrete class for control events """
 
     def __init__(self, pkt):
-        # pylint: disable=too-many-boolean-expressions
-        if isinstance(pkt, (lowlevel.Lighting1, lowlevel.Lighting2,
-                            lowlevel.Lighting3, lowlevel.Lighting4,
-                            lowlevel.Lighting5, lowlevel.Lighting6)):
-            device = LightingDevice(pkt)
-        elif isinstance(pkt, lowlevel.RollerTrol):
-            device = RollerTrolDevice(pkt)
-        elif isinstance(pkt, lowlevel.Rfy):
-            device = RfyDevice(pkt)
-        elif isinstance(pkt, lowlevel.Chime):
-            device = ChimeDevice(pkt)
-        else:
-            device = RFXtrxDevice(pkt)
+        device = get_device_from_pkt(pkt)
         super().__init__(device)
 
         self.values = {}
@@ -755,8 +770,8 @@ class PyNetworkTransport(RFXtrxTransport):
             if not data or data == '\x00':
                 continue
             pkt = bytearray(data)
-            while len(pkt) < pkt[0]:
-                data = self.sock.recv(pkt[0])
+            while len(pkt) < pkt[0]+1:
+                data = self.sock.recv(pkt[0]+1 - len(pkt))
                 pkt.extend(bytearray(data))
             _LOGGER.debug(
                 "Recv: %s",
