@@ -24,81 +24,11 @@ RFXtrx.
 # pylint: disable=C0302,R0902,R0903,R0911,R0913
 # pylint: disable= too-many-lines, too-many-statements
 
-
-def parse(data):
-    # pylint: disable=too-many-branches
-    """ Parse a packet from a bytearray """
-    if data[0] == 0 or len(data) < 2:
-        # null length packet - sometimes happens on initialization
-        return None
-
-    expected_length = data[0] + 1
-    if len(data) != expected_length:
-        return None
-
-    if data[1] == 0x01:
-        pkt = Status()
-    elif data[1] == 0x10:
-        pkt = Lighting1()
-    elif data[1] == 0x11:
-        pkt = Lighting2()
-    elif data[1] == 0x12:
-        pkt = Lighting3()
-    elif data[1] == 0x13:
-        pkt = Lighting4()
-    elif data[1] == 0x14:
-        pkt = Lighting5()
-    elif data[1] == 0x15:
-        pkt = Lighting6()
-    elif data[1] == 0x16:
-        pkt = Chime()
-    elif data[1] == 0x19:
-        pkt = RollerTrol()
-    elif data[1] == 0x1A:
-        pkt = Rfy()
-    elif data[1] == 0x20:
-        pkt = Security1()
-    elif data[1] == 0x50:
-        pkt = Temp()
-    elif data[1] == 0x4E:
-        pkt = Bbq()
-    elif data[1] == 0x4F:
-        pkt = TempRain()
-    elif data[1] == 0x51:
-        pkt = Humid()
-    elif data[1] == 0x52:
-        pkt = TempHumid()
-    elif data[1] == 0x53:
-        pkt = Baro()
-    elif data[1] == 0x54:
-        pkt = TempHumidBaro()
-    elif data[1] == 0x55:
-        pkt = Rain()
-    elif data[1] == 0x56:
-        pkt = Wind()
-    elif data[1] == 0x57:
-        pkt = UV()
-    elif data[1] == 0x59:
-        pkt = Energy1()
-    elif data[1] == 0x5A:
-        pkt = Energy()
-    elif data[1] == 0x5B:
-        pkt = Energy4()
-    elif data[1] == 0x5C:
-        pkt = Energy5()
-    elif data[1] == 0x71:
-        pkt = RfxMeter()
-    else:
-        return None
-
-    pkt.load_receive(data)
-    return pkt
-
-
 ###############################################################################
 # Packet class
 ###############################################################################
 
+
 class Packet():
     """ Abstract superclass for all low level packets """
 
@@ -1638,7 +1568,10 @@ class Rain(SensorPacket):
         0x04: 'UPM RG700',
         0x05: 'WS2300',
         0x06: 'La Crosse TX5',
-        0x07: 'Alecto'}
+        0x07: 'Alecto',
+        0x08: 'Davis',
+        0x09: 'TFA 30.3233.01'
+    }
 
     def __str__(self):
         return ("Rain [subtype={0}, seqnbr={1}, id={2}, rainrate={3}, " +
@@ -1671,15 +1604,26 @@ class Rain(SensorPacket):
         self.id2 = data[5]
         self.rainrate1 = data[6]
         self.rainrate2 = data[7]
-        self.rainrate = (self.rainrate1 << 8) + self.rainrate2
-        if self.subtype == 2:
-            self.rainrate = float(self.rainrate) / 100
+        if self.subtype in (1, 2):
+            self.rainrate = float((self.rainrate1 << 8) + self.rainrate2)
+            if self.subtype == 2:
+                self.rainrate = float(self.rainrate) / 100
         self.raintotal1 = data[8]
         self.raintotal2 = data[9]
         self.raintotal3 = data[10]
-        self.raintotal = float((self.raintotal1 << 16) +
-                               (self.raintotal2 << 8) +
-                               self.raintotal3) / 10
+
+        if self.subtype in (1, 2, 3, 4, 5, 7):
+            self.raintotal = float((self.raintotal1 << 16) +
+                                   (self.raintotal2 << 8) +
+                                   self.raintotal3) / 10
+        elif self.subtype == 6:
+            self.raintotal = 0.266 * self.raintotal3
+        elif self.subtype == 8:
+            # cartridge can be 0.01 inch rather than 0.2mm
+            self.raintotal = 0.2 * self.raintotal3
+        elif self.subtype == 9:
+            self.raintotal = 0.254 * self.raintotal3
+
         self.rssi_byte = data[11]
         self.battery = self.rssi_byte & 0x0f
         self.rssi = self.rssi_byte >> 4
@@ -1815,9 +1759,10 @@ class Wind(SensorPacket):
         self.id1 = data[4]
         self.id2 = data[5]
         self.direction = data[6] * 256 + data[7]
-        self.average_speed = (data[8] * 256.0 + data[9]) / 10.0
+        if self.subtype != 0x05:
+            self.average_speed = (data[8] * 256.0 + data[9]) / 10.0
         self.gust = (data[10] * 256.0 + data[11]) / 10.0
-        if self.subtype not in (0x06, 0x07):
+        if self.subtype in (0x04, 0x08, 0x09):
             self.temphigh = data[12]
             self.templow = data[13]
             self.temperature = float(((self.temphigh & 0x7f) << 8) +
@@ -1831,7 +1776,7 @@ class Wind(SensorPacket):
             if self.chillhigh >= 0x80:
                 self.chill = -1 * self.chill
         if self.subtype == 0x03:
-            self.battery = data[16] + 1 * 10
+            self.battery = data[16]
         else:
             self.rssi_byte = data[16]
             self.battery = self.rssi_byte & 0x0f
@@ -2014,7 +1959,7 @@ class Energy(SensorPacket):
                            (data[15] << 8) + data[16]) / 223.666
 
         if self.subtype == 0x03:
-            self.battery = data[17] + 1 * 10
+            self.battery = data[17]
         else:
             self.rssi_byte = data[17]
             self.battery = self.rssi_byte & 0x0f
@@ -2169,6 +2114,99 @@ class Energy5(SensorPacket):
                                                          self.subtype)
 
 
+###############################################################################
+# Cartelectronic class Encoder et Linky (TIC not treated - old standard)
+###############################################################################
+
+
+class Cartelectronic(SensorPacket):
+    """
+    Data class for the Cartelectronic packet type
+    """
+
+    TYPES = {0x01: 'CARTELECTRONIC_TIC',
+             0x02: 'CARTELECTRONIC_ENCODER',
+             0x03: 'CARTELECTRONIC_LINKY'}
+    """
+    Mapping of numeric subtype values to strings, used in type_string
+    """
+
+    def __str__(self):
+        return ("Cartelectronic [subtype={0}, seqnbr={1}, id={2}, " +
+                "counter1={3}, counter2={4}, " +
+                "conswatthours={5}, prodwatthours={6}, tarif_num={7}, " +
+                "voltage={8}, currentwatt={9}, teleinfo_ok={10},"
+                "battery={11}, rssi={12}]") \
+            .format(self.type_string, self.seqnbr, self.id_string,
+                    self.counter1, self.counter2,
+                    self.conswatthours, self.prodwatthours, self.tarif_num,
+                    self.voltage, self.currentwatt, self.teleinfo_ok,
+                    self.battery, self.rssi)
+
+    def __init__(self):
+        """Constructor"""
+        super().__init__()
+        self.id1 = None
+        self.id2 = None
+        self.id3 = None
+        self.id4 = None
+        self.id_combined = None
+        self.counter1 = None
+        self.counter2 = None
+        self.conswatthours = None
+        self.prodwatthours = None
+        self.tarif_num = None
+        self.voltage = None
+        self.currentwatt = None
+        self.teleinfo_ok = None
+        self.battery = None
+        self.rssi = None
+
+    def load_receive(self, data):
+        """Load data from a bytearray"""
+        self.data = data
+        self.packetlength = data[0]
+        self.packettype = data[1]
+        self.subtype = data[2]
+        self.seqnbr = data[3]
+        self.id1 = data[4]
+        self.id2 = data[5]
+        self.id3 = data[6]
+        self.id4 = data[7]
+        self.id_combined = ((self.id1 << 24) + (self.id2 << 16) +
+                            (self.id3 << 8) + self.id4)
+        if self.subtype == 0x02:
+            # Cartelectronic Encoder
+            self.counter1 = ((data[8] * pow(2, 24)) + (data[9] << 16) +
+                             (data[10] << 8) + data[11])
+            self.counter2 = ((data[12] * pow(2, 24)) + (data[13] << 16) +
+                             (data[14] << 8) + data[15])
+            self.rssi_byte = data[17]
+        elif self.subtype == 0x03:
+            # Cartelectronic Linky
+            self.conswatthours = ((data[8] * pow(2, 24)) + (data[9] << 16) +
+                                  (data[10] << 8) + data[11])
+            self.prodwatthours = ((data[12] * pow(2, 24)) + (data[13] << 16) +
+                                  (data[14] << 8) + data[15])
+            self.tarif_num = (data[16] & 0x0f)
+            self.voltage = data[17] + 200
+            self.currentwatt = (data[18] << 8) + data[19]
+            self.teleinfo_ok = not (data[20] & 0x04) == 0x04
+            self.rssi_byte = data[21]
+        self.battery = self.rssi_byte & 0x0f
+        self.rssi = self.rssi_byte >> 4
+        self._set_strings()
+
+    def _set_strings(self):
+        """Translate loaded numeric values into convenience strings"""
+        self.id_string = "{0:08x}".format(self.id_combined)
+        if self.subtype in self.TYPES:
+            self.type_string = self.TYPES[self.subtype]
+        else:
+            # Degrade nicely for yet unknown subtypes
+            self.type_string = self._UNKNOWN_TYPE.format(self.packettype,
+                                                         self.subtype)
+
 ###############################################################################
 # Chime class
 ###############################################################################
@@ -2188,11 +2226,19 @@ class Chime(Packet):
     Mapping of numeric subtype values to strings, used in type_string
     """
 
+    COMMANDS = {
+        sound: "Sound {}".format(sound)
+        for sound in range(16)
+    }
+    """
+    Mapping of a chime sound to a descriptive name.
+    """
+
     def __str__(self):
         return ("Chime [subtype={0}, seqnbr={1}, id={2}, sound={3}, " +
-                "battery={6}, rssi={7}]") \
+                "rssi={5}, cmdn={6}]") \
             .format(self.type_string, self.seqnbr, self.id_string,
-                    self.sound, self.battery, self.rssi)
+                    self.sound, self.rssi, self.cmnd_string)
 
     def __init__(self):
         """Constructor"""
@@ -2200,7 +2246,6 @@ class Chime(Packet):
         self.id1 = None
         self.id2 = None
         self.sound = None
-        self.battery = None
         self.rssi = None
         self.cmnd = None
         self.cmnd_string = None
@@ -2229,7 +2274,6 @@ class Chime(Packet):
         self.id2 = data[5]
         self.sound = data[6]
         self.rssi_byte = data[7]
-        self.battery = self.rssi_byte & 0x0f
         self.rssi = self.rssi_byte >> 4
         self._set_strings()
 
@@ -2242,9 +2286,8 @@ class Chime(Packet):
         self.id1 = id1
         self.id2 = id2
         self.sound = sound
-        self.battery = 0
         self.rssi = 0
-        self.rssi_byte = (self.rssi << 4) | self.battery
+        self.rssi_byte = (self.rssi << 4)
         self.data = bytearray([self.packetlength, self.packettype,
                                self.subtype, self.seqnbr,
                                self.id1, self.id2, self.sound,
@@ -2259,7 +2302,7 @@ class Chime(Packet):
             # Degrade nicely for yet unknown subtypes
             self.type_string = self._UNKNOWN_TYPE.format(self.packettype,
                                                          self.subtype)
-        self.cmnd_string = "Chime"
+        self.cmnd_string = self.COMMANDS.get(self.sound, "Sound")
 
 ###############################################################################
 # Security1 class
@@ -2335,6 +2378,21 @@ class Security1(SensorPacket):
         self.rssi = None
         self.security1_status_string = 'unknown'
 
+    def parse_id(self, subtype, id_string):
+        """Parse a string id into individual components"""
+        try:
+            self.packettype = 0x20
+            self.subtype = subtype
+            self.id_combined = int(id_string[:6], 16)
+            self.id1 = self.id_combined >> 16
+            self.id2 = self.id_combined >> 8 & 0xff
+            self.id3 = self.id_combined & 0xff
+            self._set_strings()
+        except ValueError as exc:
+            raise ValueError("Invalid id_string") from exc
+        if self.id_string != id_string:
+            raise ValueError("Invalid id_string")
+
     def load_receive(self, data):
         """Load data from a bytearray"""
         self.data = data
@@ -2348,10 +2406,31 @@ class Security1(SensorPacket):
         self.id_combined = (self.id1 << 16) + (self.id2 << 8) + self.id3
         self.security1_status = data[7]
         self.rssi_byte = data[8]
-        self.battery = self.rssi_byte & 0x0f
+        if self.subtype not in (0x03, 0x09, 0x0A):
+            self.battery = self.rssi_byte & 0x0f
         self.rssi = self.rssi_byte >> 4
         self._set_strings()
 
+    def set_transmit(self, subtype, seqnbr, id_combined, status):
+        """Load data from individual data fields"""
+        self.packetlength = 0x08
+        self.packettype = 0x20
+        self.subtype = subtype
+        self.seqnbr = seqnbr
+        self.id_combined = id_combined
+        self.id1 = id_combined >> 16
+        self.id2 = id_combined >> 8 & 0xff
+        self.id3 = id_combined & 0xff
+        self.security1_status = status
+        self.rssi_byte = 0
+        self.battery = 0
+        self.rssi = 0
+        self.data = bytearray([self.packetlength, self.packettype,
+                               self.subtype, self.seqnbr,
+                               self.id1, self.id2, self.id3,
+                               self.security1_status, self.rssi_byte])
+        self._set_strings()
+
     def _set_strings(self):
         """Translate loaded numeric values into convenience strings"""
         self.id_string = "{0:06x}:{1}".format(self.id_combined,
@@ -2496,7 +2575,15 @@ class RollerTrol(Packet):
     """
     Data class for the RollerTrol packet type
     """
-    TYPES = {0x00: 'RollerTrol'}
+    TYPES = {0x00: 'RollerTrol',
+             0x01: 'BlindsT1 / Hasta old',
+             0x02: 'BlindsT2 / A-OK RF01',
+             0x03: 'BlindsT3 / A-OK AC114',
+             0x04: 'BlindsT4 / Raex YR1326',
+             0x05: 'BlindsT5 / Media Mount',
+             0x06: 'BlindsT6 / DC106/Rohrmotor24-RMF/Yooda',
+             0x07: 'BlindsT7 / Forest'
+    }
     """
     Mapping of numeric subtype values to strings, used in type_string
     """
@@ -2604,3 +2691,74 @@ class RollerTrol(Packet):
                 self.cmnd_string = self.COMMANDS[self.cmnd]
             else:
                 self.cmnd_string = self._UNKNOWN_CMND.format(self.cmnd)
+
+
+PACKET_TYPES = {
+    0x01: Status,
+    0x10: Lighting1,
+    0x11: Lighting2,
+    0x12: Lighting3,
+    0x13: Lighting4,
+    0x14: Lighting5,
+    0x15: Lighting6,
+    0x16: Chime,
+    0x19: RollerTrol,
+    0x1A: Rfy,
+    0x20: Security1,
+    0x50: Temp,
+    0x4E: Bbq,
+    0x4F: TempRain,
+    0x51: Humid,
+    0x52: TempHumid,
+    0x53: Baro,
+    0x54: TempHumidBaro,
+    0x55: Rain,
+    0x56: Wind,
+    0x57: UV,
+    0x59: Energy1,
+    0x5A: Energy,
+    0x5B: Energy4,
+    0x5C: Energy5,
+    0x60: Cartelectronic,
+    0x71: RfxMeter,
+}
+
+
+def get_packet(packettype):
+    """Return a packet based on the packet type."""
+    cls = PACKET_TYPES.get(packettype)
+    if cls is None:
+        return None
+    return cls()
+
+
+def get_packet_with_id(packettype, subtype, id_string):
+    """Return a packet based on the type and identifiers."""
+    pkt = get_packet(packettype)
+    if pkt is None or not hasattr(pkt, "parse_id"):
+        return None
+    pkt.parse_id(subtype, id_string)
+    return pkt
+
+
+def parse(data):
+    """ Parse a packet from a bytearray """
+    if data[0] == 0 or len(data) < 2:
+        # null length packet - sometimes happens on initialization
+        return None
+
+    expected_length = data[0] + 1
+    if len(data) != expected_length:
+        return None
+
+    pkt = get_packet(data[1])
+    if pkt is None:
+        return None
+
+    try:
+        pkt.load_receive(data)
+    except IndexError:
+        # parsing failed due to invalid packet length
+        return None
+
+    return pkt
