@@ -15,6 +15,7 @@ class CoreTestCase(TestCase):
         self.path = '/dev/serial/...'
         global num_calbacks
         num_calbacks = 0
+        self.maxDiff = 1000
 
     def test_constructor(self):
         global num_calbacks
@@ -27,6 +28,12 @@ class CoreTestCase(TestCase):
         core.close_connection()
         self.assertFalse(core._thread.is_alive())
 
+    def test_invalid_packet(self):
+        bytes_array = bytearray([0x09, 0x11, 0xd7, 0x00, 0x01, 0x1d, 0x14, 0x02, 0x79, 0x0a])
+        core = RFXtrx.Connect(self.path, event_callback=_callback, transport_protocol=RFXtrx.DummyTransport)
+        event = core.transport.parse(bytes_array)
+        self.assertIsNone(event)
+
     def test_format_packet(self):
         # Lighting1
         core = RFXtrx.Connect(self.path, event_callback=_callback, transport_protocol=RFXtrx.DummyTransport)
@@ -183,7 +190,7 @@ class CoreTestCase(TestCase):
         bytes_array = [0x10, 0x56, 0x01, 0x03, 0x2F, 0x00, 0x00, 0xF7, 0x00, 0x20, 0x00, 0x24, 0x81, 0x60, 0x82, 0x50, 0x59]
         event = core.transport.parse(bytes_array)
         self.assertEqual(RFXtrx.SensorEvent, type(event))
-        self.assertEqual(event.__str__(),"<class 'RFXtrx.SensorEvent'> device=[<class 'RFXtrx.RFXtrxDevice'> type='WTGR800' id='2f:00'] values=[('Battery numeric', 9), ('Chill', -59.2), ('Rssi numeric', 5), ('Temperature', -35.2), ('Wind average speed', 3.2), ('Wind direction', 247), ('Wind gust', 3.6)]")
+        self.assertEqual(event.__str__(),"<class 'RFXtrx.SensorEvent'> device=[<class 'RFXtrx.RFXtrxDevice'> type='WTGR800' id='2f:00'] values=[('Battery numeric', 9), ('Rssi numeric', 5), ('Wind average speed', 3.2), ('Wind direction', 247), ('Wind gust', 3.6)]")
 
         #uv
         bytes_array = [0x09, 0x57, 0x02, 0x02, 0x64, 0x00, 0x20, 0x02, 0x3c, 0x69]
@@ -197,6 +204,18 @@ class CoreTestCase(TestCase):
         self.assertEqual(RFXtrx.SensorEvent, type(event))
         self.assertEqual(event.__str__(),"<class 'RFXtrx.SensorEvent'> device=[<class 'RFXtrx.RFXtrxDevice'> type='ELEC4, CM180i' id='2e:b2'] values=[('Battery numeric', 9), ('Count', 1), ('Current Ch. 1', 437.0), ('Current Ch. 2', 514.1), ('Current Ch. 3', 591.2), ('Rssi numeric', 6), ('Total usage', 113527617921.3023)]")
 
+        #Cartelectronic Encoder
+        bytes_array = [0x11, 0x60, 0x02, 0x5f, 0x3f, 0xfe, 0x61, 0xa3, 0x00, 0x00, 0x47, 0xd4, 0x00, 0x00, 0x00, 0x00, 0x00, 0x69]
+        event = core.transport.parse(bytes_array)
+        self.assertEqual(RFXtrx.SensorEvent, type(event))
+        self.assertEqual(event.__str__(),"<class 'RFXtrx.SensorEvent'> device=[<class 'RFXtrx.RFXtrxDevice'> type='CARTELECTRONIC_ENCODER' id='3ffe61a3'] values=[('Battery numeric', 9), ('Count', 0), ('Counter value', 18388), ('Rssi numeric', 6)]")
+       
+        #Cartelectronic Linky
+        bytes_array = [0x15, 0x60, 0x03, 0x5c, 0x2a, 0x29, 0xf2, 0x75, 0x00, 0x19, 0x8d, 0x3a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x21, 0x02, 0xb9, 0x00, 0x69]
+        event = core.transport.parse(bytes_array)
+        self.assertEqual(RFXtrx.SensorEvent, type(event))
+        self.assertEqual(event.__str__(),"<class 'RFXtrx.SensorEvent'> device=[<class 'RFXtrx.RFXtrxDevice'> type='CARTELECTRONIC_LINKY' id='2a29f275'] values=[('Battery numeric', 9), ('Count', 0), ('Counter value', 0), ('Energy usage', 697), ('Rssi numeric', 6), ('Sensor Status', True), ('Total usage', 1674554), ('Voltage', 233)]")
+
         #Lighting5
         bytes_array = bytearray(b'\x0A\x14\x00\xAD\xF3\x94\xAB'
                                 b'\x01\x01\x00\x60')
@@ -221,6 +240,12 @@ class CoreTestCase(TestCase):
         event.device.send_open(core.transport)
         event.device.send_close(core.transport)
         event.device.send_stop(core.transport)
+        event.device.send_up05sec(core.transport)
+        event.device.send_down05sec(core.transport)
+        event.device.send_up2sec(core.transport)
+        event.device.send_down2sec(core.transport)
+        event.device.send_on(core.transport)
+        event.device.send_off(core.transport)
 
         #temphumid
         bytes_array = [0x0a, 0x52, 0x01, 0x2a, 0x96, 0x03, 0x81, 0x41, 0x60, 0x03, 0x79]
@@ -232,14 +257,15 @@ class CoreTestCase(TestCase):
         bytes_array = [0x0a, 0x16, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00]
         event = core.transport.parse(bytes_array)
         self.assertEqual(RFXtrx.ControlEvent, type(event))
-        self.assertEqual(event.__str__(),"<class 'RFXtrx.ControlEvent'> device=[<class 'RFXtrx.ChimeDevice'> type='Byron SX' id='00:00'] values=[('Command', 'Chime'), ('Rssi numeric', 0), ('Sound', 0)]")
-        event.device.send_chime(core.transport, 1)
+        self.assertEqual(event.__str__(),"<class 'RFXtrx.ControlEvent'> device=[<class 'RFXtrx.ChimeDevice'> type='Byron SX' id='00:00'] values=[('Command', 'Sound 0'), ('Rssi numeric', 0), ('Sound', 0)]")
+        event.device.send_command(core.transport, 1)
 
         #security1
         bytes_array = [0x0a, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00]
         event = core.transport.parse(bytes_array)
         self.assertEqual(RFXtrx.SensorEvent, type(event))
-        self.assertEqual(event.__str__(),"<class 'RFXtrx.SensorEvent'> device=[<class 'RFXtrx.RFXtrxDevice'> type='X10 Security' id='000000:32'] values=[('Battery numeric', 0), ('Rssi numeric', 0), ('Sensor Status', 'Normal')]")
+        self.assertEqual(event.__str__(),"<class 'RFXtrx.SensorEvent'> device=[<class 'RFXtrx.SecurityDevice'> type='X10 Security' id='000000:32'] values=[('Battery numeric', 0), ('Rssi numeric', 0), ('Sensor Status', 'Normal')]")
+        event.device.send_status(core.transport, 0x01)
 
         #temp
         bytes_array = [0x0a, 0x50, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00]
